package org.bitbucket.bradleysmithllc.webserviceshubclient;

import com.informatica.wsh.*;
import org.bitbucket.bradleysmithllc.java_cl_parser.CLIEntry;
import org.bitbucket.bradleysmithllc.java_cl_parser.CLIMain;
import org.bitbucket.bradleysmithllc.java_cl_parser.CLIOption;
import org.bitbucket.bradleysmithllc.java_cl_parser.CommonsCLILauncher;
import org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.ParameterFile;
import org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.ScopeSection;

import javax.xml.namespace.QName;
import java.io.File;
import java.io.IOException;
import java.net.URL;
import java.sql.Timestamp;

@CLIEntry(
		nickName = "jcmd",
		description = "Executes an informatica workflow using the Informatica Web Services Hub.  The parameters for the workflow are pulled from a parameter file.",
		contact = "bradleysmithllc@gmail.com",
		version = "1.6.3",
		versionControl = "https://bradleysmithllc@bitbucket.org/bradleysmithllc/webserviceshubclient.git",
		documentationUrl = "https://bitbucket.org/bradleysmithllc/webserviceshubclient/wiki/Home"
)
public class JCmd
{
	private String domainName;
	private String repositoryName;
	private String integrationServiceName;
	private String userName;
	private String password;
	private String securityDomain;

	private File prm;

	private String wsdlURL;

	private int integrationServiceTimeout;

	private String folderName;
	private String workflowName;
	private String runid;

	private String taskPath;

	private boolean asynchronous;
	private boolean debug;

	@CLIOption(
			name = "dbg",
			longName = "debug",
			description = "Debug.",
			valueType = CLIOption.value_type.not_allowed
	)
	public void setDebug(boolean debug)
	{
		this.debug = debug;
	}

	@CLIOption(
			name = "rin",
			longName = "run-instance",
			description = "The run instance name.  Usually required for concurrent workflows.",
			valueType = CLIOption.value_type.required
	)
	public void setRunid(String runid)
	{
		this.runid = runid;
	}

	@CLIOption(
			name = "prm",
			longName = "parameter-file-path",
			description = "The path to the parameter file, if one is needed.  This path is relative to the caller, not the PowerCenter server.",
			valueType = CLIOption.value_type.required
	)
	public void setPrmFile(String path)
	{
		prm = new File(path);
	}

	@CLIOption(
			name = "sd",
			longName = "security-domain",
			description = "The name of the security domain.  If not supplied native will be used.  Passing 'native' will not work.",
			valueType = CLIOption.value_type.required
	)
	public void setSecurityDomain(String securityDomain)
	{
		this.securityDomain = securityDomain;
	}

	@CLIOption(
			name = "async",
			longName = "asynchronous",
			description = "If true, the command returns once the workflow is started.  Otherwise, the command blocks until the workflow completes.",
			valueType = CLIOption.value_type.not_allowed,
			defaultValue = "false"
	)
	public void setAsynchronous(boolean asynchronous)
	{
		this.asynchronous = asynchronous;
	}

	@CLIOption(
			name = "svcto",
			longName = "integration-service-timeout",
			description = "Integration Service ping timeout",
			valueType = CLIOption.value_type.required,
			defaultValue = "1000"
	)
	public void setIntegrationServiceTimeout(int integrationServiceTimeout)
	{
		this.integrationServiceTimeout = integrationServiceTimeout;
	}

	@CLIOption(
			name = "dn",
			longName = "domain-name",
			valueType = CLIOption.value_type.required,
			required = true,
			description = "The name of the Informatica Domain"
	)
	public void setDomainName(String domainName)
	{
		this.domainName = domainName;
	}

	@CLIOption(
			name = "rn",
			longName = "repository-name",
			valueType = CLIOption.value_type.required,
			required = true,
			description = "The name of the Repository Serice"
	)
	public void setRepositoryName(String repositoryName)
	{
		this.repositoryName = repositoryName;
	}

	@CLIOption(
			name = "isvc",
			longName = "integration-service",
			valueType = CLIOption.value_type.required,
			required = true,
			description = "The name of the Integration Service"
	)
	public void setIntegrationServiceName(String integrationServiceName)
	{
		this.integrationServiceName = integrationServiceName;
	}

	@CLIOption(
			name = "un",
			longName = "user-name",
			valueType = CLIOption.value_type.required,
			required = true,
			description = "The user name"
	)
	public void setUserName(String userName)
	{
		this.userName = userName;
	}

	@CLIOption(
			name = "pwd",
			longName = "password",
			valueType = CLIOption.value_type.required,
			required = true,
			description = "The password"
	)
	public void setPassword(String password)
	{
		this.password = password;
	}

	@CLIOption(
			name = "wsdl",
			longName = "wsdl-url",
			valueType = CLIOption.value_type.required,
			required = true,
			description = "The URL to the WSDL"
	)
	public void setWsdlURL(String wsdlURL)
	{
		this.wsdlURL = wsdlURL;
	}

	@CLIOption(
			name = "f",
			longName = "folder",
			valueType = CLIOption.value_type.required,
			required = true,
			description = "Repository folder name"
	)
	public void setFolderName(String folderName)
	{
		this.folderName = folderName;
	}

	@CLIOption(
			name = "wkf",
			longName = "workflow",
			valueType = CLIOption.value_type.required,
			required = true,
			description = "The workflow name"
	)
	public void setWorkflowName(String workflowName)
	{
		this.workflowName = workflowName;
	}

	@CLIOption(
			name = "tip",
			valueType = CLIOption.value_type.required,
			longName = "task-instance-path",
			description = "The task instance path execute."
	)
	public void setTaskPath(String taskPath)
	{
		this.taskPath = taskPath;
	}

	@CLIMain
	public void main() throws IOException
	{
		try
		{
			DataIntegrationService dis = new DataIntegrationService(
					new URL(wsdlURL),
					new QName("http://www.informatica.com/wsh", "DataIntegrationService")
			);

			int runId;

			DIServiceInfo diServiceInfo = new DIServiceInfo();
			diServiceInfo.setDomainName(domainName);
			diServiceInfo.setServiceName(integrationServiceName);

			SessionIdHandlerResolver sessionIdHandlerResolver = new SessionIdHandlerResolver();
			dis.setHandlerResolver(sessionIdHandlerResolver);

			DataIntegrationInterface iface = dis.getDataIntegration();

			LoginRequest loginRequest = new LoginRequest();
			loginRequest.setRepositoryDomainName(domainName);
			loginRequest.setRepositoryName(repositoryName);
			loginRequest.setUserName(userName);
			loginRequest.setPassword(password);

			TypeStartWorkflowExRequest workflowRequest = new TypeStartWorkflowExRequest();
			workflowRequest.setDIServiceInfo(diServiceInfo);
			workflowRequest.setFolderName(folderName);
			workflowRequest.setWorkflowName(workflowName);

			if (taskPath != null)
			{
				workflowRequest.setTaskInstancePath(taskPath);
			}

			if (runid != null)
			{
				workflowRequest.setWorkflowRunInstanceName(runid);
			}

			workflowRequest.setRequestMode(ETaskRunMode.NORMAL);

			if (prm != null)
			{
				ParameterArray parameterArray = new ParameterArray();

				workflowRequest.setParameters(parameterArray);

				ParameterFile pf = new ParameterFile(prm);

				if (debug)
				{
					System.out.println("Binding parameters from file: " + prm);
				}

				for (ScopeSection section : pf.getSections())
				{
					if (debug)
					{
						System.out.println("Binding parameters from section: " + section.getHeader() + " - literally '" + section.getWSHeader() + "'");
					}

					for (org.bitbucket.bradleysmithllc.webserviceshubclient.parameter.Parameter parameter : section.getParameters())
					{
						if (debug)
						{
							System.out.println("Binding parameter: " + parameter.asString());
						}

						com.informatica.wsh.Parameter wsparameter = new com.informatica.wsh.Parameter();
						wsparameter.setName(parameter.getParameterName());
						wsparameter.setValue(parameter.getParameterValue());
						wsparameter.setScope(section.getWSHeader());
						parameterArray.getParameters().add(wsparameter);
					}
				}
			}

			if (securityDomain != null)
			{
				loginRequest.setUserNameSpace(securityDomain);
			}

			String sessionId = iface.login(loginRequest);

			try
			{
				sessionIdHandlerResolver.setSessionId(sessionId);

				PingDIServerRequest pingDIServer = new PingDIServerRequest();

				pingDIServer.setDIServiceInfo(diServiceInfo);
				pingDIServer.setTimeOut(integrationServiceTimeout);

				EPingState res = iface.pingDIServer(pingDIServer);

				if (res != EPingState.ALIVE)
				{
					throw new IOException("Integration Service ping timeout");
				}

				TypeStartWorkflowExResponse resp = iface.startWorkflowEx(workflowRequest);
				runId = resp.getRunId();

				System.out.println("Running with run identifier: " + runId);
				System.out.println();
			}
			finally
			{
				iface.logout(new VoidRequest());
			}

			while (!asynchronous)
			{
				sessionId = iface.login(loginRequest);

				try
				{
					sessionIdHandlerResolver.setSessionId(sessionId);

					TypeGetWorkflowDetailsExRequest getWorkflowDetailsEx = new TypeGetWorkflowDetailsExRequest();
					getWorkflowDetailsEx.setDIServiceInfo(diServiceInfo);
					getWorkflowDetailsEx.setFolderName(workflowRequest.getFolderName());
					getWorkflowDetailsEx.setWorkflowName(workflowRequest.getWorkflowName());
					getWorkflowDetailsEx.setWorkflowRunId(runId);

					DIServerDetails status = iface.getWorkflowDetailsEx(getWorkflowDetailsEx);

					System.out.println(new Timestamp(System.currentTimeMillis()));
					EWorkflowRunStatus workflowRunStatus = status.getWorkflowDetails().get(0).getWorkflowRunStatus();
					System.out.println("Workflow status: " + workflowRunStatus.name());
					System.out.println();

					if (workflowRunStatus != EWorkflowRunStatus.RUNNING)
					{
						if (workflowRunStatus != EWorkflowRunStatus.SUCCEEDED)
						{
							throw new IOException("Bad workflow status: " + workflowRunStatus);
						}

						break;
					}
				}
				finally
				{
					iface.logout(new VoidRequest());
				}

				Thread.sleep(3000L);
			}
		}
		catch (Fault fault)
		{
			fault.printStackTrace();
			throw new IOException("Web Services Hub Failure", fault);
		}
		catch (InterruptedException e)
		{
			throw new IOException("Internal error", e);
		}
	}

	public static void main(String[] argv)
	{
		CommonsCLILauncher.mainClean(argv);
	}
}